/*
 * @Author: smallStone
 * @Date: 2018-07-02 14:43:17
 * @Last Modified by: smallStone
 * @Last Modified time: 2018-07-31 15:04:30
 * 核心函数
 */

var layer = {
    data: {},
    ArtboardScale: 1,
    allData: [],
    allLayers: 0,
    constraintFlag: false,
    /**
     * 获取需要上传的 Artboard
     * 计算图层总数
     * @param {any} target 是全部还是选择的
     * @param {any} windowObject 页面对象
     */
    getSyncArtboard: function (target) {
        var self = this,
            document = this.context.document,
            selectedObj = {},
            artloop = document.currentPage().artboards().objectEnumerator(),
            layerNSArray = document.selectedLayers().layers(),
            result = 0,
            allLayers = 0,
            allArtboardObj = [];

        if (target == 'pages') {
            // 导出 选中
            if (layerNSArray.count() == 0) {
                //没有选中的
                this.windowObject.evaluateWebScript('vm.noselection()')
                return false
            } else {
                // 通过选择的图层 上溯到画板
                for (var ns = 0; ns < layerNSArray.count(); ns++) {
                    try {
                        var artid = layerNSArray[ns].parentArtboard().objectID()
                        selectedObj[artid] = true;
                    } catch (e) {
                        log("getSyncArtboard target pages err" + e)
                    }
                }
            }
        }

        while (msArtboard = artloop.nextObject()) {
            if (msArtboard.class() == 'MSArtboardGroup') {
                if (target == "pages") {
                    //导出选中
                    for (var j in selectedObj) {
                        if (j == msArtboard.objectID()) {
                            allArtboardObj.push(msArtboard)
                            allLayers += msArtboard.children().count()
                        }
                    }
                } else {
                    //导出全部
                    allArtboardObj.push(msArtboard)
                    allLayers += msArtboard.children().count()
                }
            }
        };

        if (allArtboardObj == 0) {
            if (target == 'pages') {
                this.windowObject.evaluateWebScript('vm.noselection()')
            } else {
                this.alert('当前 pages 没有画板')
            }
            return false
        }

        var res = {
            allArtboard: new String(allArtboardObj.length).toString(),
            allLayers: new String(allLayers).toString()
        };

        this.windowObject.evaluateWebScript("vm.getallArtboard(" + JSON.stringify(res) + ")");

        this.allArtboardObj = allArtboardObj
        this.allLayers = allLayers
        return true
    },
    //初始化图层信息
    initlayer: function (context, document, sliceScale, ArtboardScale, exportSVG, amplification, windowObject, webViewRequest, rootURL) {
        this.allData = [];
        this.windowObject = windowObject;
        this.exportSVG = exportSVG
        this.amplification = amplification
        this.context = context;
        this.document = document;
        this.sliceScale = sliceScale;
        this.rootURL = rootURL;

        this.ArtboardScale = ArtboardScale;
        this.sketchVersion = NSBundle.mainBundle().infoDictionary().objectForKey("CFBundleShortVersionString")
        if (this.sketchVersion >= 49) {
            var Document = require('sketch/dom').Document
            this.Sketchdocument = Document.getSelectedDocument()
        }
        this.data.sketchVersion = this.sketchVersion

        var goon = this.getSyncArtboard(webViewRequest)
        if (!goon) return

        this.shouldClose = false;

        var allArtboardObj = this.allArtboardObj,
            allLayers = this.allLayers,
            self = this,
            idx = 0,
            artboardIndex = 0,
            layerIndex = 0;

        coscript.scheduleWithRepeatingInterval_jsFunction(0, function (interval) {

            try {
                idx++;
                windowObject.evaluateWebScript("vm.loadLayer=" + idx);
                windowObject.evaluateWebScript("vm.processLayer()");

                var artboard = allArtboardObj[artboardIndex],
                    layer = artboard.children()[layerIndex];


                // 初始化 Artbaord
                if (!self.allData[artboardIndex]) {

                    var artInfo = self.getArtboard(artboard)

                    self.allData[artboardIndex] = artInfo
                }


                layerIndex++;


                //获取 layer
                if (artboard != layer) {

                    self.getInfo(artboard, layer, self.allData[artboardIndex])
                    // console.log("self.allData[artboardIndex]",self.allData[artboardIndex])
                }

                if (layerIndex >= artboard.children().length) {

                    layerIndex = 0;
                    artboardIndex++;
                }

                if (idx >= allLayers) {

                    self.shouldClose = true
                    //循环结束
                    for (var i = 0; i < self.allData.length; i++) {
                        windowObject.evaluateWebScript("vm.uploadJSON(" + JSON.stringify(self.allData[i].data) + ", '" + self.allData[i].base64 + "')");
                    }
                    windowObject.evaluateWebScript("vm.cocoaDone='true'");
                }

            } catch (e) {
                log("loop err" + e);
                self.shouldClose = true
            }


            if (self.shouldClose) {

                self.threadDictionary.removeObjectForKey(self.identifier)
                interval.cancel()
            }
        })
    },
    getInfo: function (artboard, layer, data) {
        try {
            var msReact = artboard.absoluteRect()
            var ruleX = msReact.x()
            var ruleY = msReact.y()
            var sliceScale = this.sliceScale
            var artWidth = Math.round(msReact.width() * 10) / 10
            var artHeight = Math.round(msReact.height() * 10) / 10
            var opt = {
                    layer: layer,
                    data: data,
                    ruleX: ruleX,
                    ruleY: ruleY,
                    sliceScale: sliceScale,
                    artWidth: artWidth,
                    artHeight: artHeight,
                    topOffset: null,
                    bottomOffset:null,
                }
            this.exportLayer(opt)
        } catch (e) {
            log("getInfo err" + e)
        }


    },
    getArtboard: function (msArtboard) {
        try {

            this.data = {};
            var document = this.document,
                ArtboardScale = this.ArtboardScale,
                sliceScale = this.sliceScale,
                context = this.context;

            if (msArtboard instanceof MSArtboardGroup) {

                //检查objectID 是否是UUID
                if (!/[A-F0-9]{8}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{4}-[A-F0-9]{12}/.test(msArtboard.objectID())) {
                    msArtboard.generateObjectID();
                }

                var ArtboardID = msArtboard.objectID(),
                    msReact = msArtboard.absoluteRect(),
                    ruleX = msReact.x(),
                    ruleY = msReact.y();

                //计算比例 此处为获取各个元素的大小

                var originalWidth = Math.round(msReact.width() * 10) / 10,
                    artWidth = Math.round(msReact.width() * 10) / 10,
                    artHeight = Math.round(msReact.height() * 10) / 10;


                var base64 = this.exportArtboardAsBitmap(msArtboard,ArtboardScale,this.amplification);

                //最终push到标注页面的最终数据
                this.data = {
                    sketchVersion: this.sketchVersion,
                    skVersion:this.toStringValue(this.sketchVersion),
                    ArtboardScale: this.toStringValue(ArtboardScale),
                    ArtboardID: this.toStringValue(ArtboardID),
                    info: [],
                    pageName: this.toStringValue(this.emoji(document.currentPage().name())),
                    sliceScale: sliceScale,
                    flowData: []
                };
                //最终push到标注页面的最终数据的info字段
                //画板信息为第一个数组元素.

                this.data['info'].push({
                    name: this.toStringValue(this.emoji(msArtboard.name())),
                    left: 0,
                    top: 0,
                    width: artWidth,
                    height: artHeight,
                    position_x: msReact.x() / 4,
                    position_y: msReact.y() / 4,
                });

                if (this.sketchVersion >= 51) {
                    if (msArtboard.presetDictionary()) {
                    var scrollInfo = {
                        viewportHeight: this.toStringValue(msArtboard.presetDictionary()["height"]),
                        viewportWidth: this.toStringValue(msArtboard.presetDictionary()["width"]),
                        sliceScale: this.toStringValue(sliceScale),
                        artboardScale: this.toStringValue(ArtboardScale),
                        artboardWidth: this.toStringValue(artWidth),
                        artboardHeight: this.toStringValue(artHeight),
                    }
                    this.data.scrollInfo = scrollInfo
                }
                }

                if (msArtboard.hasBackgroundColor()) {
                    this.data['info'][0].fills = [{
                        color: this.getLayerColor(msArtboard.backgroundColor()),
                        type: "color"
                    }]
                };

                // try {
                //     if (msArtboard.exportOptions().exportFormats().count() > 0) {
                //         var bitmapBase64Data = this.exportLayerAsBitmap(msArtboard, sliceScale);
                //         var image = {
                //             "bitmap": this.toStringValue(bitmapBase64Data['bitmap']),
                //             "width": this.toStringValue(bitmapBase64Data['width']),
                //             "height": this.toStringValue(bitmapBase64Data['height'])
                //         }
                //         if (this.exportSVG) {
                //             var vectorBase64 = this.exportLayerAsVector(msArtboard, msArtboard.name);
                //             image.svg = this.toStringValue(vectorBase64);
                //         }
                //         this.data['info'][0].image = image;
                //     }
                // } catch (error) {
                //     console.log("export artboard asset error", error);
                // }



                // this.exportLayer(msArtboard, ruleX, ruleY, artWidth, artHeight, sliceScale);

                return {
                    data: this.data,
                    base64: this.toStringValue(base64),
                }
            }
        } catch (error) {
            log("getArtboard err" + error)
        }
    },
    exportLayer: function (opt) {
        try {

            var msLayer = opt.layer
            var ruleX = opt.ruleX
            var ruleY = opt.ruleY
            var artWidth = opt.artWidth
            var artHeight = opt.artHeight
            var sliceScale = opt.sliceScale
            var saveGroup = opt.saveGroup || false
            var symbolID = opt.symbolID || null
            var isGroup = false
            var data = opt.data
            var hasConstraint = opt.hasConstraint
            var topOffset =  opt.topOffset
            var bottomOffset = opt.bottomOffset


            // 此处可以判断opt.layer.class()是否是MSLayerGroup类型,如果是,则为group组类型
            // library类型oc类为MSSymbolInstance
            // symbol类型的oc类同样为MSSymbolInstance 区别方法暂时未知.

            // 判断是否为组类型
            if (opt.layer.class() == "MSLayerGroup") {
                isGroup = true;
            }


            var msGroup = msLayer.parentGroup();
            var layerProperties = this.getLayerProperties(msLayer);
            var parent = this.getLayerProperties(msLayer.parentGroup());

            var layer = {};

            // log("是否是组" + isGroup)
            // log("是否是切图" + parent.isExportable)


            // if (!layerProperties.isVisible ||
            //     msGroup.class() == "MSShapeGroup" ||
            //     layerProperties.layerType == "symbol-master" ||
            //     layerProperties.layerType == 'artboard-group' ||
            //     parent.isExportable ||
            //     (layerProperties.layerType == 'layer-group' && !layerProperties.isExportable && !msLayer.flow() && !saveGroup) ||
            //     msLayer.name().indexOf('-x-') == 0
            // ) {
            //     //忽略-x-开头的图层
            //     return;
            // };


            var layerRect;
            if (!layerProperties.isRenderable) {
                layerRect = msLayer.absoluteInfluenceRect(),
                left = (Math.round((layerRect.origin.x - ruleX) * 10) / 10),
                top = (Math.round((layerRect.origin.y - ruleY) * 10) / 10),
                width = (Math.round(layerRect.size.width * 10) / 10),
                height = (Math.round(layerRect.size.height * 10) / 10);
            } else {
                layerRect = msLayer.absoluteRect(),
                left = (Math.round((layerRect.x() - ruleX) * 10) / 10),
                top = (Math.round((layerRect.y() - ruleY) * 10) / 10),
                width = (Math.round(layerRect.width() * 10) / 10),
                height = (Math.round(layerRect.height() * 10) / 10);

            }

            var autoLayoutRulers = 0;

            var isAutolayout = false;
            var constraint = {}

            if (msLayer.hasFixedHeight() || (this.constraintFlag == true && layerProperties.layerType != 'layer-group')) {
                autoLayoutRulers = autoLayoutRulers + 1;
                constraint.height = height;
            }
            if (msLayer.hasFixedWidth() || (this.constraintFlag == true && layerProperties.layerType != 'layer-group')) {
                autoLayoutRulers = autoLayoutRulers + 1;
                constraint.width = width;
            }
            if (msLayer.hasFixedBottom() || (this.constraintFlag == true && layerProperties.layerType != 'layer-group')) {
                autoLayoutRulers = autoLayoutRulers + 1;
                constraint.bottom = artHeight - height - top;
            }
            if (msLayer.hasFixedTop() || (this.constraintFlag == true && layerProperties.layerType != 'layer-group')) {
                autoLayoutRulers = autoLayoutRulers + 1;
                constraint.top = top;
            }
            if (msLayer.hasFixedRight() || (this.constraintFlag == true && layerProperties.layerType != 'layer-group')) {
                autoLayoutRulers = autoLayoutRulers + 1;
                constraint.right = artWidth - left - width;
            }
            if (msLayer.hasFixedLeft() || (this.constraintFlag == true && layerProperties.layerType != 'layer-group')) {
                autoLayoutRulers = autoLayoutRulers + 1;
                constraint.left = left;
            }

            if (autoLayoutRulers == 4 || (this.constraintFlag == true && layerProperties.layerType != 'layer-group')) {
                isAutolayout = true;
                layer.constraint = constraint;
            }

            if (hasConstraint !== undefined && layerProperties.layerType != 'layer-group') {
                this.constraintFlag = false;
            }




            //mask child
            if (layerProperties.isMaskChildLayer) {
                //忽略以下图层
                if (left > this.maskRect.width + this.maskRect.left) {
                    return;
                };
                if (top > this.maskRect.top + this.maskRect.height) {
                    return;
                };
                if (top + height < this.maskRect.top) {
                    return;
                };
                if (left + width < this.maskRect.left) {
                    return;
                };

                //超出的裁掉
                if (left + width > this.maskRect.left + this.maskRect.width) {
                    width = this.maskRect.left + this.maskRect.width - left;
                };
                if (top + height > this.maskRect.top + this.maskRect.height) {
                    height = this.maskRect.top + this.maskRect.height - top;
                };
                if (top < this.maskRect.top) {
                    height = top + height - this.maskRect.top;
                    top = this.maskRect.top;
                };
                if (left < this.maskRect.left) {
                    width = width + left - this.maskRect.left;
                    left = this.maskRect.left;
                };
            };


            if (symbolID) {
                layer.symbolID = this.toStringValue(symbolID);
            }


            if (this.sketchVersion >= 51) {
                if (msLayer.isFixedToViewport() == "1" || msLayer.isFixedToViewport() == true) {
                    if (top == 0) {
                        layer.isFixedToViewport = true;
                        layer.fixedToViewportType = "top";
                        if (data.data.scrollInfo.topOffset == null) {
                            data.data.scrollInfo.topOffset = height;
                        } else {
                            if (data.data.scrollInfo.topOffset <= height) {
                                data.data.scrollInfo.topOffset = height;
                            }
                        }
                    }
                    if (top + height >= artHeight) {
                        layer.isFixedToViewport = true;
                        layer.fixedToViewportType = "bottom";
                        if (data.data.scrollInfo.bottomOffset == null) {
                            data.data.scrollInfo.bottomOffset = height;
                        } else {
                            if (data.data.scrollInfo.bottomOffset <= height) {
                                data.data.scrollInfo.bottomOffset = height;
                            }
                        }
                    }
                }
            }

            if (!layerProperties.isVisible ||
                msGroup.class() == "MSShapeGroup" ||
                layerProperties.layerType == "symbol-master" ||
                layerProperties.layerType == 'artboard-group' ||
                parent.isExportable ||
                (layerProperties.layerType == 'layer-group' && !layerProperties.isExportable && !msLayer.flow() && !saveGroup) ||
                msLayer.name().indexOf('-x-') == 0
            ) {
                //忽略-x-开头的图层
                return;
            };
            layer.name = this.toStringValue(this.emoji(msLayer.name()));
            layer.type = layerProperties.layerType;
            layer.top = top;
            layer.left = left;
            layer.height = height;
            layer.width = width;
            layer.sharedStyle = this.getSharedStyle(msLayer)
            layer.isAutolayout = isAutolayout;
            //超出部分裁剪掉
            if (layer.width + layer.left > artWidth) {
                layer.width = artWidth - layer.left;
            };
            if (layer.left < 0) {
                layer.width = layer.width - Math.abs(layer.left);
                layer.left = 0;
            };
            if (layer.height + layer.top > artHeight) {
                layer.height = artHeight - layer.top;
            };
            if (layer.top < 0) {
                layer.height = layer.height - Math.abs(layer.top);
                layer.top = 0;
            };

            //49的原型功能

            try {
                if (this.sketchVersion >= 49 && msLayer.flow()) {
                    if (layer.type == 'hotspot') {
                        var flowData = {
                            targetId: this.toStringValue(msLayer.flow().destinationArtboardID()),
                            animationType: this.getAnmitationType(msLayer)
                        }
                        console.log("flowData",flowData)
                    } else {
                        var objid = msLayer.objectID()
                        // var flowData = this.Sketchdocument.getLayerWithID(objid).flow.toJSON()
                        var flowData = {
                            type: "Flow",
                            targetId: this.toStringValue(msLayer.flow().destinationArtboardID()),
                            animationType: this.getAnmitationType(msLayer)
                        }
                    }

                    layer.flow = flowData;
                    log("flowData" + flowData)

                    this.data['flow'] = true;
                    this.data['flowData'].push({
                        width: layer.width,
                        height: layer.height,
                        top: layer.top,
                        left: layer.left,
                        flow: flowData
                    })
                }

            } catch (e) {
                log("49 flow err" + e)
            }

            try {
                if (layerProperties.isExportable ||
                    (msLayer.exportOptions().exportFormats().count() > 0 && layerProperties.layerType == "symbol") ||
                    (msLayer.symbolMaster().exportOptions().exportFormats().count() > 0 && layerProperties.layerType == "symbol")
                ) {

                    // 检查 symbol 是否是切图
                    var inslayer = msLayer
                    if (layerProperties.layerType == "symbol" && msLayer.symbolMaster().exportOptions().exportFormats().count() > 0) {
                        layerProperties.isExportable = true
                        var overrideValues = inslayer.overrideValues();
                        if (overrideValues.length == 0) {
                            inslayer = msLayer.symbolMaster()
                        }
                    }

                    layer.image = {};
                    if (layerProperties.layerType == "bitmap") {
                        try {
                            var bitmapBase64Data = this.exportLayerAsBitmap(inslayer, sliceScale);
                            layer.image['bitmap'] = this.toStringValue(bitmapBase64Data.bitmap);
                            layer.image['width'] = this.toStringValue(bitmapBase64Data.width);
                            layer.image['height'] = this.toStringValue(bitmapBase64Data.height);
                        } catch (error) {
                            log('bitmap Base64 err' + e);
                        }


                    } else {
                        try {
                            var bitmapBase64 = this.exportLayerAsBitmap(inslayer, sliceScale).bitmap;
                            layer.image['bitmap'] = this.toStringValue(bitmapBase64);
                            layer.image['width'] = this.toStringValue(this.exportLayerAsBitmap(inslayer, sliceScale).width);
                            layer.image['height'] = this.toStringValue(this.exportLayerAsBitmap(inslayer, sliceScale).height);
                        } catch (e) {
                            log('bitmapBase64 err' + e);
                        }
                        try {
                            if (this.exportSVG) {
                                var vectorBase64 = this.exportLayerAsVector(inslayer, layer.name);
                                log("vectorBase64" + vectorBase64);
                                if (vectorBase64 != false) {
                                    if (vectorBase64.type == "url") {
                                        layer.image['svgUrl'] = this.toStringValue(vectorBase64.value);
                                    } else {
                                        layer.image['svg'] = this.toStringValue(vectorBase64.value);
                                    }

                                }
                            }
                        } catch (e) {
                            log('exportSVG err' + e);
                        }



                    }
                    layer.exportable = true;
                } else {
                    layer.exportable = false;
                };
            } catch (e) {
                log('export err' + e);
                // log('layerProperties' + layerProperties);
            }

            //此处判断是否是切图,如果是,就忽略图层的样式.如果不是,就把各种基础样式加上

            if (layerProperties.isRenderable && layerProperties.layerType != "symbol" && layerProperties.layerType != "symbol-master") {
                //基础样式
                var layerStyle = msLayer.style();

                layer.opacity = Math.round(layerStyle.contextSettings().opacity() * 10000) / 100;

                var radius = this.getLayerRadius(msLayer);
                if (radius) layer.radius = radius;

                var borders = this.getLayerBorders(msLayer, layerStyle);
                if (borders.length > 0) layer.borders = borders;

                var dash = this.getborderOptions(msLayer, layerStyle);
                if (dash.length > 0) layer.dash = dash;

                var fills = this.getLayerFills(msLayer, layerStyle);
                if (fills.length > 0) layer.fills = fills;

                var shadow = this.getLayerShadow(msLayer, layerStyle);
                if (shadow.length > 0) layer.shadow = shadow;

                var blur = this.getLayerBlur(msLayer, layerStyle);
                if (blur) layer.blur = blur;
            }

            if (layerProperties.layerType == "text") {
                try {
                    var tree = msLayer.attributedStringValue().treeAsDictionary();
                    var textAligns = ["left", "right", "center", "justify", "left"];
                    layer.font = {
                        content: this.toStringValue(this.emoji(tree.objectForKey("text").replace(/\s+/g, " "))),
                        font: this.toStringValue(msLayer.fontPostscriptName()),
                        size: msLayer.fontSize(),
                        transform: this.toStringValue(tree["attributes"][0]["MSAttributedStringTextTransformAttribute"]),
                        line: msLayer.lineHeight() > 0 ? msLayer.lineHeight() : msLayer.font().defaultLineHeightForFont(),
                        spacing: msLayer.characterSpacing() == null ? this.toStringValue(0) : this.toStringValue(Math.round(msLayer.characterSpacing() * 100) / 100),
                        align: textAligns[msLayer.textAlignment()],
                        color: this.getLayerColor(msLayer.textColor()),
                        verticalAlignment: msLayer.verticalAlignment(),
                        displayName: this.toStringValue(msLayer.font().displayName())
                    };

                    var _this = this;
                    var styles = [];
                    if (this.sketchVersion >= 48) {
                        var attributes = msLayer.attributedString().treeAsDictionary().value.attributes;
                        attributes.forEach(function (attr) {
                            var value;
                            if (attr.MSAttributedStringColorAttribute.value.indexOf('rgba(') > -1) {
                                value = attr.MSAttributedStringColorAttribute.value
                            } else {
                                value = _this.hextorgba(attr.MSAttributedStringColorAttribute.value)
                            }

                            var lineHeight, content, family, letterSpacing, fontSize, paragraphSpacing, displayName, length, location, strikethrough, underline, transform;
                            if (attr.NSParagraphStyle && attr.NSParagraphStyle.style.maximumLineHeight && attr.NSParagraphStyle.style.maximumLineHeight != 0) {
                                lineHeight = attr.NSParagraphStyle.style.maximumLineHeight;
                            } else {
                                lineHeight = msLayer.lineHeight() || msLayer.font().defaultLineHeightForFont();
                            };

                            if (attr.NSStrikethrough) {
                                strikethrough = attr.NSStrikethrough
                            }

                            if (attr.NSUnderline) {
                                underline = attr.NSUnderline
                            }

                            if (attr.length) {
                                length = attr.length
                            }

                            if (attr.location) {
                                location = attr.location
                            }

                            if (attr.NSFont && attr.NSFont.attributes && attr.NSFont.attributes.NSFontNameAttribute) {
                                family = attr.NSFont.attributes.NSFontNameAttribute;
                                displayName = attr.NSFont.name;
                            } else {
                                family = attr.NSFont.family;
                                displayName = attr.NSFont.name;
                            };

                            if (attr.NSKern) {
                                letterSpacing = Math.round(attr.NSKern * 100) / 100;
                            } else {
                                letterSpacing = 0;
                            };

                            if (attr.NSFont && attr.NSFont.attributes && attr.NSFont.attributes.NSFontSizeAttribute) {
                                fontSize = attr.NSFont.attributes.NSFontSizeAttribute;
                            } else {
                                fontSize = msLayer.fontSize();
                            };
                            if (attr.NSParagraphStyle.style.paragraphSpacing) {
                                paragraphSpacing = attr.NSParagraphStyle.style.paragraphSpacing;
                            }
                            if (attr.MSAttributedStringTextTransformAttribute) {
                                transform = attr.MSAttributedStringTextTransformAttribute;
                            }
                            styles.push({
                                content: _this.toStringValue(attr.objectForKey("text").replace(/\s+/g, " ")),
                                font: _this.toStringValue(family),
                                size: _this.toStringValue(fontSize),
                                line: _this.toStringValue(lineHeight),
                                spacing: _this.toStringValue(letterSpacing),
                                transform: _this.toStringValue(transform),
                                align: _this.toStringValue(textAligns[attr.NSParagraphStyle.style.alignment]),
                                paragraphSpacing: _this.toStringValue(paragraphSpacing),
                                displayName: _this.toStringValue(displayName),
                                length: _this.toStringValue(length),
                                location: _this.toStringValue(location),
                                strikethrough: _this.toStringValue(strikethrough),
                                underline: _this.toStringValue(underline),
                                color: {
                                    value: _this.toStringValue(value)
                                }
                            });
                        });
                    } else {
                        var tree = msLayer.attributedStringValue().treeAsDictionary();
                        var attributes = tree.attributes.objectEnumerator();
                        while (attribute = attributes.nextObject()) {
                            var color = attribute.NSColor.color;
                            color = color.substring(color.indexOf("(") + 1, color.indexOf(")")).split(",");
                            var r = Math.round(color[0] * 255),
                                g = Math.round(color[1] * 255),
                                b = Math.round(color[2] * 255),
                                a = Math.round(color[3]),
                                value = `rgba(${r},${g},${b},${a})`;

                            var lineHeight, content, family, letterSpacing, fontSize, paragraphSpacing, displayName, length, location, strikethrough, underline, transform;
                            if (attribute.NSParagraphStyle && attribute.NSParagraphStyle.style.maximumLineHeight && attribute.NSParagraphStyle.style.maximumLineHeight != 0) {
                                lineHeight = attribute.NSParagraphStyle.style.maximumLineHeight;
                            } else {
                                lineHeight = msLayer.lineHeight() || msLayer.font().defaultLineHeightForFont();
                            };

                            if (attr.NSStrikethrough) {
                                strikethrough = attr.NSStrikethrough
                            }

                            if (attr.NSUnderline) {
                                underline = attr.NSUnderline
                            }

                            if (attr.length) {
                                length = attr.length
                            }

                            if (attr.location) {
                                location = attr.location
                            }

                            if (attribute.NSFont && attribute.NSFont.attributes && attribute.NSFont.attributes.NSFontNameAttribute) {
                                family = attribute.NSFont.attributes.NSFontNameAttribute;
                                displayName = attr.NSFont.name;
                            } else {
                                family = attribute.NSFont.family;
                                displayName = attr.NSFont.name;
                            };

                            if (attribute.NSKern) {
                                letterSpacing = Math.round(attribute.NSKern * 100) / 100;
                            } else {
                                letterSpacing = 0;
                            };

                            if (attribute.NSFont && attribute.NSFont.attributes && attribute.NSFont.attributes.NSFontSizeAttribute) {
                                fontSize = attribute.NSFont.attributes.NSFontSizeAttribute;
                            } else {
                                fontSize = msLayer.fontSize();
                            };
                            if (attr.NSParagraphStyle.style.paragraphSpacing) {
                                paragraphSpacing = attr.NSParagraphStyle.style.paragraphSpacing;
                            }
                            if (attr.MSAttributedStringTextTransformAttribute) {
                                transform = attr.MSAttributedStringTextTransformAttribute;
                            }
                            styles.push({
                                content: this.toStringValue(attribute.objectForKey("text").replace(/\s+/g, " ")),
                                font: this.toStringValue(family),
                                size: this.toStringValue(fontSize),
                                line: this.toStringValue(lineHeight),
                                spacing: this.toStringValue(letterSpacing),
                                align: this.toStringValue(textAligns[attribute.NSParagraphStyle.style.alignment]),
                                displayName: _this.toStringValue(displayName),
                                length: _this.toStringValue(length),
                                location: _this.toStringValue(location),
                                strikethrough: _this.toStringValue(strikethrough),
                                underline: _this.toStringValue(underline),
                                transform: _this.toStringValue(transform),
                                color: {
                                    r: this.toStringValue(r),
                                    g: this.toStringValue(g),
                                    b: this.toStringValue(b),
                                    a: this.toStringValue(a),
                                    value: this.toStringValue(value)
                                },
                                paragraphSpacing: _this.toStringValue(paragraphSpacing)
                            });
                        }
                    }
                    log("styles" + styles);
                    if (styles.length > 0) {
                        layer.font.styles = styles;
                    };

                } catch (e) {
                    log("text err" + e)
                }
            }

            if (msLayer.hasClippingMask()) {
                this.maskObjectID = msGroup.objectID();
                this.maskRect = {
                    width: layer.width,
                    height: layer.height,
                    top: layer.top,
                    left: layer.left
                };
            } else if (this.maskObjectID != msGroup.objectID() || msLayer.shouldBreakMaskChain()) {
                this.maskObjectID = undefined;
                this.maskRect = undefined;
            };

            if (layerProperties.layerType == "symbol" &&
                msLayer.symbolMaster().children().count() > 1 &&
                !layerProperties.isExportable
            ) {
                if (layer.isAutolayout == false) {
                    this.constraintFlag = false;
                } else {
                    this.constraintFlag = true;
                }


                var symbolID = msLayer.symbolMaster().objectID();
                var tempSymbol = msLayer.duplicate();
                var temp;
                if (this.sketchVersion >= 53) {
                    temp = tempSymbol.detachStylesAndReplaceWithGroupRecursively(false);
                } else {
                    temp = tempSymbol.detachByReplacingWithGroup();
                    temp.resizeToFitChildrenWithOption(0);
                }


                var tempLoop = temp.children().objectEnumerator();

                while (tempSymbolLayer = tempLoop.nextObject()) {

                    var opt = {
                        layer: tempSymbolLayer,
                        ruleX: ruleX,
                        ruleY: ruleY,
                        artWidth: artWidth,
                        artHeight: artHeight,
                        sliceScale: sliceScale,
                        saveGroup: true,
                        symbolID: symbolID,
                        data: data,
                        hasConstraint: layer.isAutolayout
                    }

                    this.exportLayer(opt);
                }

                symbolID = null;
                var container = temp.parentGroup();
                if (container) container.removeLayer(temp);
            } else {
                data.data['info'].push(layer);
            };
        } catch (e) {
            log("exportLayer err" + e)
        }
    }
}