/*
 * @Author: smallStone
 * @Date: 2018-07-03 15:47:29
 * @Last Modified by: smallStone
 * @Last Modified time: 2018-07-31 13:33:15
 * style 相关
 */
var style = {
    emoji: function (str) {
        var emojiRegExp = new RegExp("(?:[\u2700-\u27bf]|(?:\ud83c[\udde6-\uddff]){2}|[\ud800-\udbff][\udc00-\udfff]|[\u0023-\u0039]\ufe0f?\u20e3|\u3299|\u3297|\u303d|\u3030|\u24c2|\ud83c[\udd70-\udd71]|\ud83c[\udd7e-\udd7f]|\ud83c\udd8e|\ud83c[\udd91-\udd9a]|\ud83c[\udde6-\uddff]|[\ud83c[\ude01-\ude02]|\ud83c\ude1a|\ud83c\ude2f|[\ud83c[\ude32-\ude3a]|[\ud83c[\ude50-\ude51]|\u203c|\u2049|[\u25aa-\u25ab]|\u25b6|\u25c0|[\u25fb-\u25fe]|\u00a9|\u00ae|\u2122|\u2139|\ud83c\udc04|[\u2600-\u26FF]|\u2b05|\u2b06|\u2b07|\u2b1b|\u2b1c|\u2b50|\u2b55|\u231a|\u231b|\u2328|\u23cf|[\u23e9-\u23f3]|[\u23f8-\u23fa]|\ud83c\udccf|\u2934|\u2935|[\u2190-\u21ff])", "g");
        return str.replace(
            emojiRegExp,
            function (match) {
                var u = "";
                for (var i = 0; i < match.length; i++) {
                    if (!(i % 2)) {
                        u += "&#" + match.codePointAt(i)
                    }
                }


                return u;
            });
    },
    hextorgba: function (hex) {
        var c = '';
        if (/^#([A-Fa-f0-9]{3}){1,2}$/.test(hex)) {
            c = hex.substring(1).split('');
            if (c.length == 3) {
                c = [c[0], c[0], c[1], c[1], c[2], c[2]];
            }
            c = '0x' + c.join('');
            return 'rgba(' + [(c >> 16) & 255, (c >> 8) & 255, c & 255].join(',') + ',1)';
        }
    },
    // 导出画板图
    exportArtboardAsBitmap: function (artboard, scale, amplification) {
        try {
            var path = NSTemporaryDirectory() + artboard.objectID() + '.png';
            var request = MSExportRequest.new();
            var document = this.document;
            request.rect = artboard.absoluteInfluenceRect();
            request.format = 'png';
            request.scale = scale;
            if (amplification) {
                request.scale = scale * 2;
            }
            request.includeArtboardBackground = true;
            document.saveExportRequest_toFile(request, path)

            //read file content and convert to base64
            var dataURL = NSURL.fileURLWithPath(path);
            var dataImg = NSData.alloc().initWithContentsOfURL(dataURL);
            var base64 = dataImg.base64EncodedStringWithOptions(0);
            var url = NSURL.fileURLWithPath(path);
            NSFileManager.defaultManager().removeItemAtURL_error(url, nil)
            return base64;
        } catch (error) {
            log("exportArtboardAsBitmap" + error);
            return false;
        }

    },
    // 切图转换base64
    exportLayerAsBitmap: function (layer, scale) {
        var slice;
        // var sliceRect = null;
        var document = this.document;
        var imageData = {};

        // var absoluteRect = layer.absoluteRect();
        var exportName = NSTemporaryDirectory() + layer.objectID() + '.png';

        // sliceRect = NSMakeRect(absoluteRect.x(), absoluteRect.y(), absoluteRect.width(), absoluteRect.height());
        slice = MSExportRequest.exportRequestsFromExportableLayer(layer).firstObject()
        // [
        // 	[MSExportRequest exportRequestsFromExportableLayer: layer] firstObject
        // ];
        // slice.page = document.currentPage();
        slice.format = 'png';
        slice.scale = scale;

        document.saveArtboardOrSlice_toFile(slice, exportName);
        // [document saveArtboardOrSlice: slice toFile: exportName];

        var url = NSURL.fileURLWithPath(exportName),
            data = NSData.alloc().initWithContentsOfURL(url),
            // [NSData alloc] initWithContentsOfURL: url
            base64 = data.base64EncodedStringWithOptions(0);

        log("111111" + scale)
        // NSFileManager.defaultManager().removeItemAtURL_error(url, nil)
        // var image = NSImage.alloc().initWithData(data);
        // 此处获取图片的实际尺寸
        var rep = NSBitmapImageRep.imageRepWithData(data);
        imageData.bitmap = base64;
        imageData.width = rep.pixelsWide()/4;
        imageData.height = rep.pixelsHigh()/4;
        return imageData;
    },

    exportLayerAsVector: function (layer, name) {
        var slice;
        // var sliceRect = null;
        var document = this.document;

        // var secretUrl = NSUserDefaults.standardUserDefaults().objectForKey("secretUrl");
        var cloud_type = JSON.parse(NSUserDefaults.standardUserDefaults().objectForKey("select")).selected.cloud_type;

        // var absoluteRect = layer.absoluteRect();
        var exportName = NSTemporaryDirectory() + layer.objectID() + '.svg';

        // sliceRect = NSMakeRect(absoluteRect.x(), absoluteRect.y(), absoluteRect.width(), absoluteRect.height());
        slice = MSExportRequest.exportRequestsFromExportableLayer(layer).firstObject()
        // [
        // 	[MSExportRequest exportRequestsFromExportableLayer: layer] firstObject
        // ];
        slice.page = document.currentPage();
        slice.format = 'svg';
        // [document saveArtboardOrSlice: slice toFile: exportName];
        document.saveArtboardOrSlice_toFile(slice, exportName)
        var url = NSURL.fileURLWithPath(exportName)
            // [NSURL fileURLWithPath: exportName];
        var fileSize = NSFileManager.defaultManager().attributesOfItemAtPath_error(exportName,nil).fileSize();

        // 这里是判断大小是否超过阀值,超过直接忽略
        //
        // if (fileSize > 10000000) {
        //     NSFileManager.defaultManager().removeItemAtURL_error(url, nil)
        //     return false
        // }
        var returnValue = {};
        if (cloud_type == "bos") {
            if (fileSize > 10000000) {
                NSFileManager.defaultManager().removeItemAtURL_error(url, nil)
                return false
            } else {
                var data = NSString.alloc().initWithContentsOfURL_encoding_error(url, NSUTF8StringEncoding, nil);
                // [
                // 	[NSString alloc] initWithContentsOfURL: url encoding: NSUTF8StringEncoding error: nil
                // ];
                NSFileManager.defaultManager().removeItemAtURL_error(url, nil)
                // [
                // 	[NSFileManager defaultManager] removeItemAtURL: url error: nil
                // ];
                returnValue["type"] = "base64"
                returnValue["value"] = data
                return returnValue;
            }
        } else {
            // 大于阀值直接走新的上传方法
            if (fileSize > 1000000) {
                var obj = Lanhu.alloc().init();
                var urlString = obj.uploadWithRootURL_ObjectID(this.rootURL,layer.objectID())
                log("url" + urlString)
                NSFileManager.defaultManager().removeItemAtURL_error(url, nil)
                if (urlString && urlString != "") {
                    returnValue["type"] = "url"
                    returnValue["value"] = urlString
                    return returnValue
                } else {
                    return false
                }
            } else {
                var data = NSString.alloc().initWithContentsOfURL_encoding_error(url, NSUTF8StringEncoding, nil);
                // [
                // 	[NSString alloc] initWithContentsOfURL: url encoding: NSUTF8StringEncoding error: nil
                // ];
                NSFileManager.defaultManager().removeItemAtURL_error(url, nil)
                // [
                // 	[NSFileManager defaultManager] removeItemAtURL: url error: nil
                // ];
                returnValue["type"] = "base64"
                returnValue["value"] = data
                return returnValue;
            }
        }
    },
    getLayerProperties: function (layer) {
        var isVisible = true,
            isLocked = false,
            hasSlice = false,
            isEmpty = false,
            isMaskChildLayer = false;
        var layerTemp = layer;
        while (layerTemp.class() != "MSArtboardGroup" && layerTemp.class() != "MSSymbolMaster") {
            var group = layerTemp.parentGroup();

            if (!layerTemp.isVisible()) {
                isVisible = false;
            };

            if (layerTemp.isLocked()) {
                isLocked = true;
            };

            if (layerTemp.exportOptions().exportFormats().count() > 0) {
                hasSlice = true;
            };

            if (this.maskObjectID && group.objectID() == this.maskObjectID && !layerTemp.shouldBreakMaskChain()) {
                isMaskChildLayer = true;
            };

            if (layerTemp.class() == "MSTextLayer" && layerTemp.isEmpty()) {
                isEmpty = true;
            };

            layerTemp = group;
        }

        var layerType = "shape",
            isExportFormatsAvailable = layer.exportOptions().exportFormats().count() > 0;

        //get type of layer
        if (layer.class() == "MSTextLayer") {
            layerType = "text";
        } else if (layer.class() == "MSSymbolInstance") {
            layerType = "symbol";
        } else if (layer.class() == "MSSymbolMaster") {
            layerType = "symbol-master";
        } else if (layer.class() == "MSSliceLayer") {
            layerType = "slice";
        } else if (layer.class() == "MSLayerGroup") {
            layerType = "layer-group";
        } else if (layer.class() == "MSArtboardGroup") {
            layerType = "artboard-group";
        } else if (layer.class() == "MSBitmapLayer") {
            layerType = "bitmap";
        } else if (layer.class() == "MSHotspotLayer") {
            layerType = "hotspot";
        } else if (layer.class() == "MSShapePathLayer") {
            layerType = "path";
        };
        //

        // 猜测: 标记是否是切图
        var isExportable = layerType != "artboard-group" && isExportFormatsAvailable;
        // log("name " + layer.name())
        // log("isExportable " + isExportable)
        // log(" ")

        var isRenderable = layer.class() == "MSTextLayer" ||
            layer.class() == "MSShapeGroup" ||
            layer.class() == "MSBitmapLayer" ||
            layer.class() == "MSSymbolInstance" ||
            layer.class() == "MSRectangleShape" ||
            layer.class() == "MSOvalShape" ||
            layer.class() == "MSShapePathLayer" ||
            layer.class() == "MSTriangleShape" ||
            layer.class() == "MSStarShape" ||
            layer.class() == "MSPolygonShape";

        isRenderable = isRenderable && !isEmpty && !hasSlice;

        return {
            isVisible: isVisible,
            isLocked: isLocked,
            hasSlice: hasSlice,
            isMaskChildLayer: isMaskChildLayer,
            isEmpty: isEmpty,
            layerType: layerType,
            isExportFormatsAvailable: isExportFormatsAvailable,
            isExportable: isExportable,
            isRenderable: isRenderable
        }
    },
    getLayerMaskRect: function (layer) {
        var layer = this.extend(layer, {});
        layer.maxX = layer.x + layer.width;
        layer.maxY = layer.y + layer.height;
        var mask = this.extend(this.maskRect, {});
        mask.maxX = mask.x + mask.width;
        mask.maxY = mask.y + mask.height;
        var x = layer.x;
        var y = layer.y;
        var width = layer.width;
        var height = layer.height;
        var dx = 0;
        var dy = 0;

        var isLayerMaskOverlap = !(
            layer.maxX <= mask.x ||
            layer.x >= mask.maxX ||
            layer.y >= mask.maxY ||
            layer.maxY <= mask.y
        );

        if (isLayerMaskOverlap) {
            if (layer.x < mask.x) {
                x = mask.x;
                dx = mask.x - layer.x;
            };

            if (layer.y < mask.y) {
                y = mask.y;
                dy = mask.y - layer.y;
            };

            if (layer.maxX > mask.maxX) {
                width = width - (layer.maxX - mask.maxX) - dx;
            } else {
                width = width - dx;
            };

            if (layer.maxY > mask.maxY) {
                height = height - (layer.maxY - mask.maxY) - dy;
            } else {
                height = height - dy;
            };

            return {
                x: x,
                y: y,
                width: width,
                height: height
            }
        } else {
            return false
        }

    },
    toStringValue: function (NSStr) {
        var config = arguments.length > 1 ? arguments[1] : null;
        var str = new String(NSStr).toString();

        if (config && config.escapeLine) {
            str = str.replace(/\\-/g, '-');
        };

        if (config && config.encode) {
            str = encodeURIComponent(str);
        };
        return str;
    },
    getLayerRect: function (rect, referenceRect) {
        if (referenceRect) {
            return {
                x: rect.x() - referenceRect.x(),
                y: rect.y() - referenceRect.y(),
                width: rect.width(),
                height: rect.height()
            };
        };

        return {
            x: rect.x(),
            y: rect.y(),
            width: rect.width(),
            height: rect.height()
        };
    },
    getLayerColor: function (color) {
        var r = Math.round(color.red() * 255),
            g = Math.round(color.green() * 255),
            b = Math.round(color.blue() * 255),
            a = Math.round(color.alpha() * 100) / 100,
            value = 'rgba(' + r + ',' + g + ',' + b + ',' + a + ')';
        return {
            r: r,
            g: g,
            b: b,
            a: a,
            value: value
        };
    },
    getLayerGradient: function (msLayer, gradient) {
        try {
            var colorStops = [],
                gradientTypes = ["linear", "radial", "angular"],
                colorStopsLoop = gradient.stops().objectEnumerator();
            while (colorStop = colorStopsLoop.nextObject()) {
                colorStops.push({
                    color: this.getLayerColor(colorStop.color()),
                    position: colorStop.position()
                });
            };

            return {
                type: gradientTypes[gradient.gradientType()],
                from: {
                    x: parseFloat(gradient.from().x),
                    y: parseFloat(gradient.from().y)
                },
                to: {
                    x: parseFloat(gradient.to().x),
                    y: parseFloat(gradient.to().y)
                },
                colorStops: colorStops
            };
        } catch (e) {

            log("getLayerGradient" + e)

        };
        return {};
    },
    getLayerRadius: function (msLayer) {
        try {
            if (this.sketchVersion >= 49) {
                if (this.sketchVersion >= 52) {

                    var radius1 = msLayer.points().objectAtIndex(0).cornerRadius();
                    var radius2 = msLayer.points().objectAtIndex(1).cornerRadius();
                    var radius3 = msLayer.points().objectAtIndex(2).cornerRadius();
                    var radius4 = msLayer.points().objectAtIndex(3).cornerRadius();
                    if (radius1 == 0 && radius2 == 0 && radius3 == 0 && radius4 == 0) {
                        return false;
                    } else {
                        if (radius1 == radius2 && radius2 == radius3 && radius3 == radius4) {
                            return [radius1];
                        } else {
                            return [radius1, radius2, radius3, radius4]
                        }
                    };
                } else {
                    if (msLayer.layers && msLayer.layers().firstObject().fixedRadius) {
                    var radius1 = msLayer.layers().firstObject().points().objectAtIndex(0).cornerRadius();
                    var radius2 = msLayer.layers().firstObject().points().objectAtIndex(1).cornerRadius();
                    var radius3 = msLayer.layers().firstObject().points().objectAtIndex(2).cornerRadius();
                    var radius4 = msLayer.layers().firstObject().points().objectAtIndex(3).cornerRadius();
                    if (radius1 == 0 && radius2 == 0 && radius3 == 0 && radius4 == 0) {
                        return false;
                    } else {
                        if (radius1 == radius2 && radius2 == radius3 && radius3 == radius4) {
                            return [radius1];
                        } else {
                            return [radius1, radius2, radius3, radius4]
                        }
                    };
                    }
                }

            } else {
                if (msLayer.layers && msLayer.layers().firstObject().fixedRadius) {
                    var radius1 = msLayer.layers().firstObject().path().points().objectAtIndex(0).cornerRadius();
                    var radius2 = msLayer.layers().firstObject().path().points().objectAtIndex(1).cornerRadius();
                    var radius3 = msLayer.layers().firstObject().path().points().objectAtIndex(2).cornerRadius();
                    var radius4 = msLayer.layers().firstObject().path().points().objectAtIndex(3).cornerRadius();
                    if (radius1 == 0 && radius2 == 0 && radius3 == 0 && radius4 == 0) {
                        return false;
                    } else {
                        if (radius1 == radius2 && radius2 == radius3 && radius3 == radius4) {
                            return [radius1];
                        } else {
                            return [radius1, radius2, radius3, radius4]
                        }
                    };
                }
            }

        } catch (e) {

            log("getLayerRadius" + e)

        };
        return 0;
    },
    getLayerBorders: function (msLayer, layerStyle) {
        var layerBorders = [],
            borderPositions = ["中心边框", "内边框", "外边框"],
            layerBordersLoop = layerStyle.borders().objectEnumerator();

        try {
            //loop fills and calculate
            while (border = layerBordersLoop.nextObject()) {
                if (border.isEnabled()) {
                    if (border.fillType() == 0) {
                        layerBorders.push({
                            type: "color",
                            position: borderPositions[border.position()],
                            thickness: border.thickness(),
                            color: this.getLayerColor(border.color())
                        });
                    } else if (border.fillType() == 1) {
                        layerBorders.push({
                            type: "gradient",
                            position: borderPositions[border.position()],
                            thickness: border.thickness(),
                            gradient: this.getLayerGradient(msLayer, border.gradient())
                        });
                    }
                }
            }
        } catch (e) {

            log("getLayerBorders"+ e)

        };

        return layerBorders;
    },
    getborderOptions: function (msLayer, layerStyle) {
        var loop = layerStyle.borderOptions().dashPattern().objectEnumerator(),
            dashPattern = [];
        try {
            while (dash = loop.nextObject()) {
                dashPattern.push(this.toStringValue(dash))
            }
        } catch (e) {
            log("getborderOptions" + e);
        };
        return dashPattern;
    },
    getLayerFills: function (msLayer, layerStyle) {
        var layerFills = [],
            layerFillsLoop = layerStyle.fills().objectEnumerator();

        try {
            //loop fills and calculate
            while (fill = layerFillsLoop.nextObject()) {
                if (fill.isEnabled()) {
                    if (fill.fillType() == 0) {
                        layerFills.push({
                            type: "color",
                            color: this.getLayerColor(fill.color())
                        });
                    } else if (fill.fillType() == 1) {
                        layerFills.push({
                            type: "gradient",
                            gradient: this.getLayerGradient(msLayer, fill.gradient())
                        });
                    }
                }
            }
        } catch (e) {

            log("getLayerFills" + e);

        };

        return layerFills;
    },
    getLayerBlur: function (msLayer, layerStyle) {
        //模糊
        var blurObj = {},
            blurType = {
                0: '高斯模糊',
                1: '动感模糊',
                2: '径向模糊'
            };
        try {
            var blur = layerStyle.blur();
            if (blur && blur.isEnabled()) {
                blurObj = {
                    type: blurType[blur.type()],
                    radius: Math.round(blur.radius())
                };
                return blurObj;
            } else {
                return false;
            }
        } catch (e) {
            log("getLayerBlur" + e);
            return false;
        }

    },
    getLayerShadow: function (msLayer, layerStyle) {
        var layerShadow = [],
            layerShadowLoop = layerStyle.shadows().objectEnumerator(),
            layerInnerShadowLoop = layerStyle.innerShadows().objectEnumerator();

        try {
            //loop shadow and calculate
            while (shadow = layerShadowLoop.nextObject()) {
                if (shadow.isEnabled()) {
                    layerShadow.push({
                        type: shadow instanceof MSStyleShadow ? "外阴影" : "内阴影",
                        offsetX: shadow.offsetX(),
                        offsetY: shadow.offsetY(),
                        blurRadius: shadow.blurRadius(),
                        spread: shadow.spread(),
                        color: this.getLayerColor(shadow.color())
                    });
                }
            }
            //loop inner shadow and calculate
            while (shadow = layerInnerShadowLoop.nextObject()) {
                if (shadow.isEnabled()) {
                    layerShadow.push({
                        type: shadow instanceof MSStyleShadow ? "外阴影" : "内阴影",
                        offsetX: shadow.offsetX(),
                        offsetY: shadow.offsetY(),
                        blurRadius: shadow.blurRadius(),
                        spread: shadow.spread(),
                        color: this.getLayerColor(shadow.color())
                    });
                }
            }
        } catch (e) {

            log("getLayerShadow" + e)

        };

        return layerShadow;
    },
    getAnmitationType: function (msLayer) {
        try {
            var type = msLayer.flow().animationType()
            var typeObj = {
                '-1': 'none',
                '0': 'slideFromRight',
                '1': 'slideFromLeft',
                '2': 'slideFromBottom',
                '3': 'slideFromTop',
            }
            return typeObj[type]
        } catch (e) {
            log('getAnmitationType' + e)
        }
        return ''
    },
    getSharedStyle: function (msLayer) {
        try {
            var sharedStyles = [];
            var sharedStyleID
            if (this.sketchVersion >= 52) {
                if (!msLayer.sharedStyle()) {
                    return null
                }
                sharedStyleID = msLayer.sharedStyle().objectID();
            } else {
                sharedStyleID = msLayer.style().sharedObjectID();
            }



            if (sharedStyleID != null) {
                if (this.sketchVersion >= 52) {
                    if (msLayer.class() == "MSTextLayer") {
                        sharedStyles = this.document.documentData().allTextStyles()
                    } else {
                        sharedStyles = this.document.documentData().allLayerStyles();
                    };
                    // var styles = sharedStyles.objects();
                    for (var i = 0; i < sharedStyles.count(); i++) {
                        var style = sharedStyles.objectAtIndex(i);
                        var styleID = this.toStringValue(style.objectID());

                        if (styleID == sharedStyleID) {
                            //shared style found
                            return {
                                name: this.toStringValue(style.name()),
                                sid: styleID
                            }
                        }
                    }

                    if (msLayer.class() == "MSTextLayer") {
                        sharedStyles = this.document.documentData().layerTextStyles()
                    } else {
                        sharedStyles = this.document.documentData().layerStyles();
                    };
                    var styles = sharedStyles.objects();
                    for (var i = 0; i < styles.count(); i++) {
                        var style = styles.objectAtIndex(i);
                        var styleID = this.toStringValue(style.objectID());

                        if (styleID == sharedStyleID) {
                            //shared style found
                            return {
                                name: this.toStringValue(style.name()),
                                sid: styleID
                            }
                        }
                    }
                } else {
                    if (msLayer.class() == "MSTextLayer") {
                        sharedStyles = this.document.documentData().layerTextStyles()
                    } else {
                        sharedStyles = this.document.documentData().layerStyles();
                    };
                    var styles = sharedStyles.objects();
                    for (var i = 0; i < styles.count(); i++) {
                        var style = styles.objectAtIndex(i);
                        var styleID = this.toStringValue(style.objectID());

                        if (styleID == sharedStyleID) {
                            //shared style found
                            return {
                                name: this.toStringValue(style.name()),
                                sid: styleID
                            }
                        }
                    }
                }
            }
        } catch (e) {

            log("getSharedStyle err" + e)
        };
        return null;
    },
}